@@ -24,7 +24,7 @@ class Playlist(gtk.ListStore):
         gtk.ListStore.__init__(self, object)
         
         # shuffled positions
-        self._shuffled_pos = None
+        self._permutation = None
         
         if playlist is not None:
             self.update(playlist)        
@@ -38,18 +38,21 @@ class Playlist(gtk.ListStore):
     def shuffle(self, shuffle):
         """shuffle/ unshuffle the playlist"""
         if shuffle:
-            self._shuffled_pos = range(0, len(self))
+            self._permutation = range(0, len(self))
             # build mapping
-            random.shuffle(self._shuffled_pos)
-            self.reorder(self._shuffled_pos)
+            random.shuffle(self._permutation)
         else:
             # build reverse mapping
-            rev_map = range(0, len(self))
+            rev_perm = range(0, len(self))
             for i in xrange(0, len(self)):
-                rev_map[self._shuffled_pos[i]] = i
-                
-            self.reorder(rev_map)
-            self._shuffled_pos = None    
+                rev_perm[self._permutation[i]] = i
+            
+            self._permutation = rev_perm
+            
+        self.reorder(self._permutation)
+    
+    def get_new_pos(self, i):
+        return self._permutation[i]
     
     def __getitem__(self, k):
         return gtk.ListStore.__getitem__(self, k)[0]
