@@ -5,7 +5,7 @@ import time
 import urllib2
 import sys
 
-from youamp.ui.window import Window
+from youamp.ui.window import Window, Menu
 from youamp.ui.about import AboutDialog
 from youamp.ui.preferences import Preferences
 
@@ -20,7 +20,7 @@ if IS_MAEMO:
     import osso
     
     osso_c = osso.Context("net.madman2k.youamp", VERSION, False)
-
+        
 class Controller:
     def __init__(self):
         check_db()
@@ -29,9 +29,9 @@ class Controller:
         self._player = Player(self._config)
         self._library = Library()
         self._window = Window(self._player, self._config, self._library)
-        self._lastfm_connected = False
         self._banner = None
-        
+        self._scrobbler = None
+    
         # programm
         program = hildon.Program()
         program.add_window(self._window)
@@ -41,9 +41,8 @@ class Controller:
    
         # last.fm submission
         if self._config["lastfm-user"] != "":
-            try:    
-                scrobbler.login(self._config["lastfm-user"], self._config["lastfm-pass"], client=("you", VERSION))
-                self._lastfm_connected = True
+            try:
+                self._scrobbler = scrobbler.Scrobbler(self._config["lastfm-user"], self._config["lastfm-pass"], ("you", VERSION))
             except (urllib2.URLError, scrobbler.AuthError), e:
                 sys.stderr.write("lastfm.login: %s\n" % e)
             
@@ -59,54 +58,26 @@ class Controller:
         self._preferences = None
                     
         # menu
-        menu = gtk.Menu()
+        menu = Menu(self._config)
         self._window.set_menu(menu)
-        
-        # view submenu
-        view = gtk.MenuItem(_("View"))
-        menu.append(view)
-        
-        view_sub = gtk.Menu()
-        view.set_submenu(view_sub)
-        
-        search = gtk.RadioMenuItem(None, _("Search"))
-        search.connect("activate", lambda caller: self._window.view.search_mode())
-        view_sub.append(search)
-        
-        browse = gtk.RadioMenuItem(search, _("Browse"))
-        browse.connect("activate", lambda caller: self._window.view.browse_mode())
-        view_sub.append(browse)
-    
-        # order submenu
-        order = gtk.MenuItem(_("Order"))
-        menu.append(order)
-        
-        order_sub = gtk.Menu()
-        order.set_submenu(order_sub)
-        
-        self._order_menu = dict()
-        self._order_menu["artist"] = gtk.RadioMenuItem(None, _("Artist"))
-        self._order_menu["date"] = gtk.RadioMenuItem(self._order_menu["artist"], _("Date added"))
-        self._order_menu["playcount"] = gtk.RadioMenuItem(self._order_menu["artist"], _("Playcount"))
 
-        for k in ("artist", "date", "playcount"):
-            order_sub.append(self._order_menu[k])
-     
-        prefs = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES)
-        prefs.connect("activate", self._show_preferences)
-        menu.append(prefs)
-        
-        about = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
-        about.connect("activate", self._show_about_dialog)
-        menu.append(about)
-        menu.show_all()
+        self._refresh_playlist()
+                
+        if self._config["shuffle"]:
+            self._player.playlist.shuffle(True)
 
         # restore config
         if self._player.tracks() > 0:
             self._player.load_track()
+            self._config.notify("pos")
+
+        # menu signals
+        menu.prefs.connect("activate", self._show_preferences)
+        menu.about.connect("activate", self._show_about_dialog)
         
-        if not self._config["shuffle"]:
-            self._order_menu[self._config["order-by"]].set_active(True)
+        menu.order["artist"].connect("activate", lambda caller: self._set_list_order("artist"))
+        menu.order["date"].connect("activate", lambda caller: self._set_list_order("date"))
+        menu.order["playcount"].connect("activate", lambda caller: self._set_list_order("playcount"))
 
         # signals
         self._window.connect("destroy", self.quit)
@@ -114,14 +85,7 @@ class Controller:
         
         self._player.connect("song-played", self._on_song_played)
         self._config.notify_add("music-folder", self._refresh_library)
-        
-        if self._lastfm_connected:
-            self._player.connect("song-changed", self._now_playing)
-
-        # order signals
-        self._order_menu["artist"].connect("activate", lambda caller: self._set_list_order("artist"))
-        self._order_menu["date"].connect("activate", lambda caller: self._set_list_order("date"))
-        self._order_menu["playcount"].connect("activate", lambda caller: self._set_list_order("playcount"))
+        self._player.connect("song-changed", self._now_playing)
 
         self._library.connect("update-complete", self._on_library_updated)
 
@@ -185,32 +149,33 @@ class Controller:
         if not song.has_key("duration"):
             return
         
-        if played < song["duration"]/2 or played < 240 or song["duration"] < 30:
+        if (played < song["duration"]/2 and played < 240) or song["duration"] < 30:
             return
         
         self._library.increment_played(song.uri)
         
-        if self._lastfm_connected:
+        if self._scrobbler.is_connected():
             try:
-                scrobbler.submit(song["artist"], song["title"], int(time.time()), album=song["album"], length=song["duration"])
+                self._scrobbler.submit(song["artist"], song["title"], int(time.time()), album=song["album"], length=song["duration"])
             except urllib2.URLError, e:
                 sys.stderr.write("lastfm.submit: %s\n" % e)
 
     def _now_playing(self, caller, song):
-        try:
-            scrobbler.now_playing(song["artist"], song["title"], song["album"])
-        except urllib2.URLError, e:
-            sys.stderr.write("lastfm.now_playing: %s\n" % e)
+        if self._scrobbler.is_connected():
+            try:
+                self._scrobbler.now_playing(song["artist"], song["title"], song["album"])
+            except urllib2.URLError, e:
+                sys.stderr.write("lastfm.now_playing: %s\n" % e)
         
     def _show_about_dialog(self, caller):
         if self._about_dialog is None:
-            self._about_dialog = AboutDialog()
+            self._about_dialog = AboutDialog(self._window)
         
         self._about_dialog.show()
     
     def _show_preferences(self, caller):
         if self._preferences is None:
-            self._preferences = Preferences(self._window, self._config, self._lastfm_connected)
+            self._preferences = Preferences(self._window, self._config, self._scrobbler)
             self._preferences.show_all()
     
         self._preferences.show()
@@ -249,9 +214,9 @@ class Controller:
         gtk.main()
         
     def quit(self, caller=None):        
-        if self._lastfm_connected:
+        if self._scrobbler.is_connected():
             try:
-                scrobbler.flush()
+                self._scrobbler.flush()
             except urllib2.URLError, e:
                 sys.stderr.write("lastfm.submit: %s\n" % e)
         
