@@ -113,7 +113,7 @@ class Library(GObject):
                     filelist.append(uri)
 
         return filelist
- 
+
     def _update(self, folder):    
         # local connection, so we can run as thread
         conn = sqlite3.connect(db_file)
@@ -171,9 +171,13 @@ class Library(GObject):
             if song["title"] == "None" and \
                song["artist"] == "None" and \
                song["album"] == "None":
-                    fname = uri.rindex("/")
-                    fnend = uri.rindex(".")
+                    fname = self.uri.rindex("/")
+                    fnend = self.uri.rindex(".")
                     song["title"] = uri[fname+1:fnend]
+            else:
+                # sanitise tags
+                for k in ("title", "artist", "album"):
+                    song[k] = song[k].strip()
             
             mtime = os.path.getmtime(uri[7:].replace("%25", "%")) # strip file:// and unescape
             
