@@ -21,10 +21,11 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>."""
 
 class AboutDialog(gtk.AboutDialog):
-    def __init__(self):
+    def __init__(self, parent):
         gtk.AboutDialog.__init__(self)
         
         #gtk.about_dialog_set_url_hook(self.on_url)
+        self.set_transient_for(parent)
         
         self.set_name("YouAmp")
         self.set_version(VERSION)
