@@ -5,11 +5,26 @@ import hildon
 from youamp.ui.browser import Browser
 from youamp.ui import NOTHING_SELECTED, ARTIST_SELECTED, ALBUM_SELECTED
 
+class BrowseButton(gtk.Button):
+    def __init__(self):
+        gtk.Button.__init__(self)
+        
+        self.set_size_request(200, -1)
+        
+        self.label = gtk.Label()
+        self.label.set_ellipsize(pango.ELLIPSIZE_END)
+        
+        self.add(self.label)
+        self.unset_flags(gtk.CAN_FOCUS)
+    
+    def set_text(self, txt):
+        self.label.set_text(txt)
+
 class SearchView(gtk.VBox):
     def __init__(self, config, player, library):
         gtk.VBox.__init__(self)
         self.set_spacing(5)
-        
+
         self._config = config
         self._library = library
         self._player = player
@@ -26,29 +41,19 @@ class SearchView(gtk.VBox):
         
         # Browse Button
         self._browse_button = gtk.Button(_("Browse"))
-        self._browse_button.connect("clicked", self._on_browse_click)
+        self._browse_button.connect("clicked", self._show_artists)
         self._browse_button.set_size_request(100, -1)
         self._browse_button.unset_flags(gtk.CAN_FOCUS)
         navi.pack_start(self._browse_button, expand=False)
         
         # Navi Artist Button
-        self._artist = gtk.Button()
-        self._artist.connect("clicked", self._on_artist_click)
-        self._artist.set_size_request(200, -1)
-        label = gtk.Label()
-        label.set_ellipsize(pango.ELLIPSIZE_END)
-        self._artist.add(label)
-        self._artist.unset_flags(gtk.CAN_FOCUS)
+        self._artist = BrowseButton()
+        self._artist.connect("clicked", self._show_albums)
         navi.pack_start(self._artist, expand=False)
         
         # Navi Album Button
-        self._album = gtk.Button()
-        self._album.connect("clicked", self._on_album_click)
-        self._album.set_size_request(200, -1)
-        label = gtk.Label()
-        label.set_ellipsize(pango.ELLIPSIZE_END)
-        self._album.unset_flags(gtk.CAN_FOCUS)
-        self._album.add(label)
+        self._album = BrowseButton()
+        self._album.connect("clicked", self._select_album)
         navi.pack_start(self._album, expand=False)
         
         # Navi Search Entry
@@ -72,21 +77,25 @@ class SearchView(gtk.VBox):
         self._scroll.set_shadow_type(gtk.SHADOW_IN)
         hildon.hildon_helper_set_thumb_scrollbar(self._scroll, True)
         
-        self._playlist = PlaylistView(player.playlist, self._config)
-        self._playlist.connect("row-activated", self._on_row_activated)
-        
+        self._playlist = PlaylistView(player.playlist, self._config, self._player)        
         self._browser = Browser(self._config, self._library)
         
         self._scroll.add(self._playlist)
 
         self.pack_start(self._scroll)
 
-        self.show_all()
+        self.show_all() 
  
         # Restore View
         if self._config["is-browser"]:
-            self._artist.get_child().set_text(self._config["search-artist"])
-            self._album.get_child().set_text(self._config["search-album"])
+            a = self._config["search-artist"]
+            txt = a if a != "" else _("All Artists")
+            self._artist.set_text(txt)   
+            
+            a = self._config["search-album"]
+            txt = a if a !=  "" else _("All Albums")                
+            self._album.set_text(txt)
+
             self._browser._state = ARTIST_SELECTED | ALBUM_SELECTED
             self._search_label.hide()
             self._search_entry.hide()
@@ -94,12 +103,8 @@ class SearchView(gtk.VBox):
             self._browse_button.hide()
             self._artist.hide()
             self._album.hide()
-        
-        self._restore_playlist()
-        
-        self._player.connect("song-changed", self._on_pos_changed)
-        #self._config.notify_add("pos", self._on_pos_changed)
-        
+
+   
         # browser callbacks
         self._config.notify_add("search-artist", self._on_search_artist)
         self._config.notify_add("search-album", self._on_search_album)
@@ -108,24 +113,8 @@ class SearchView(gtk.VBox):
         self._config["shuffle"] = not self._config["shuffle"]
         self._player.playlist.shuffle(self._config["shuffle"])
         
-    def _restore_playlist(self):
-        if self._config["is-browser"]:
-            if self._config["search-artist"] == "":
-                self._artist.get_child().set_text(_("All Artists"))
-            
-            if self._config["search-album"] == "":
-                self._album.get_child().set_text(_("All Albums"))
+        self._config["pos"] = self._player.playlist.get_new_pos(self._config["pos"])
         
-        playlist = self._library.get_tracks(self._config)
-
-        self._player.playlist.update(playlist)
-        
-        if self._config["shuffle"]:
-            self._player.playlist.shuffle(True)
-        
-        self._playlist.set_cursor(self._config["pos"])
-
-
     def _browse_complete(self):        
         self._search_entry.set_text("")
         self._config["is-browser"] = True
@@ -163,7 +152,7 @@ class SearchView(gtk.VBox):
             self._artist.show()
             self._album.show()
         else:
-            self._on_browse_click()
+            self._show_artists()
     
     def search_mode(self):
         self._browse_button.hide()
@@ -186,39 +175,33 @@ class SearchView(gtk.VBox):
             self._scroll.add(self._browser)
 
     def _on_search_artist(self, client, cxn_id, entry, data):
-        try:
-            val = entry.get_value().get_string()
-        except AttributeError:
-            print "error"
-        
+        val = entry.get_value().get_string()
+
         val = val if val != "" else _("All Artists")
         
-        self._artist.get_child().set_text(val)
+        self._artist.set_text(val)
         self._artist.show()
 
         self._album.hide()
         print "search-artist changed"
 
     def _on_search_album(self, client, cxn_id, entry, data):
-        try:
-            val = entry.get_value().get_string()
-        except AttributeError:
-            print "error"
-        
+        val = entry.get_value().get_string()
+  
         val = val if val != "" else _("All Albums")
         
-        self._album.get_child().set_text(val)
+        self._album.set_text(val)
         self._album.show()
         
         self._browse_complete()
         print "search-album changed"
-    
-    def _on_album_click(self, caller):        
+
+    def _select_album(self, caller):        
         if not (self._browser._state & ALBUM_SELECTED):
             self._browser._state |= ALBUM_SELECTED
             self._config.notify("search-album")
         
-    def _on_artist_click(self, caller):
+    def _show_albums(self, caller):
         if not (self._browser._state & ALBUM_SELECTED):
             self._config.notify("search-artist")
 
@@ -227,21 +210,18 @@ class SearchView(gtk.VBox):
         
         self._browser._state = ARTIST_SELECTED
 
-    def _on_browse_click(self, caller=None):
+    def _show_artists(self, caller=None):
         self._show_browser()
         self._browser.show_artists()
         self._browser._state = NOTHING_SELECTED
 
-    def _on_row_activated(self, caller, path, column):
-        self._player.goto_pos(path[0])
-    
-    def _on_pos_changed(self, *args):
-        self._playlist.set_cursor(self._config["pos"])
-
 class PlaylistView(gtk.TreeView):
-    def __init__(self, model, config):        
+    def __init__(self, model, config, player):        
         gtk.TreeView.__init__(self, model)
-    
+        
+        self._config = config
+        self._player = player
+        
         self.set_rules_hint(True)
         self.set_headers_visible(True)
         self.set_fixed_height_mode(True)
@@ -255,11 +235,19 @@ class PlaylistView(gtk.TreeView):
             col = gtk.TreeViewColumn(cols[i], cell)
             col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             col.set_fixed_width(200)
-            col.set_cell_data_func(cell, self.data_func, keys[i])
+            col.set_cell_data_func(cell, self._data_func, keys[i])
             self.append_column(col)
+        
+        # Signals
+        self._config.notify_add("pos", self._on_pos_changed)
+        self.connect("row-activated", self._on_row_activated)
 
-        self.set_cursor(config["pos"])
-    
-    def data_func(self, col, cell, model, itr, key):
+    def _on_pos_changed(self, *args):
+        self.set_cursor(self._config["pos"])
+ 
+    def _on_row_activated(self, caller, path, column):
+        self._player.goto_pos(path[0])
+
+    def _data_func(self, col, cell, model, itr, key):
         cell.set_property("text", model[itr][key])
 
